"""
Functionality relating to individual colours.
"""
from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Any, Tuple


@dataclass(frozen=True)
class Colour:
    """A colour with three channels; red, green, and blue."""

    red: int
    green: int
    blue: int
    alpha: int = 255

    @property
    def rgb(self) -> Tuple[int, int, int]:
        """Get the colour as a 3-tuple of red, green, and blue."""
        return self.red, self.green, self.blue

    @property
    def rgba(self) -> Tuple[int, int, int, int]:
        """Get the colour as a 4-tuple of red, green, blue, and alpha."""
        return self.red, self.green, self.blue, self.alpha

    @property
    def hex(self) -> str:
        """Get the colour as a lowercase hex string."""
        if self.alpha < 255:
            return f"{self.red:02x}{self.green:02x}{self.blue:02x}{self.alpha:02x}"
        return f"{self.red:02x}{self.green:02x}{self.blue:02x}"

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, Colour):
            raise ValueError("Cannot check equality with non-colour types.")

        return self.hex == other.hex

    @classmethod
    def from_hex(cls, hex_string: str) -> Colour:
        """Create a colour from hex string."""
        if len(hex_string) not in (6, 8):
            raise ValueError("Hex string must be 6 or 8 characters long.")

        num_groups = 3 if len(hex_string) == 6 else 4
        match = re.match(r"([\da-fA-F]{2})" * num_groups, hex_string)
        if match is None:
            raise ValueError("Hex string has an invalid format.")

        components = (int(col, 16) for col in match.groups())
        return Colour(*components)

    def opacity(self, opacity: float) -> Colour:
        """Return a new colour with the given opacity."""
        if not 0 <= opacity <= 1:
            raise ValueError("Opacity must be between 0 and 1.")

        return Colour(self.red, self.green, self.blue, int(opacity * 255))



"""
Rich themes for all Catppuccin flavours.
"""
from rich.theme import Theme

from catppuccin.flavour import Flavour


def _make_theme(flavour: Flavour) -> Theme:
    return Theme(
        {
            "rosewater": f"#{flavour.rosewater.hex}",
            "flamingo": f"#{flavour.flamingo.hex}",
            "pink": f"#{flavour.pink.hex}",
            "mauve": f"#{flavour.mauve.hex}",
            "red": f"#{flavour.red.hex}",
            "maroon": f"#{flavour.maroon.hex}",
            "peach": f"#{flavour.peach.hex}",
            "yellow": f"#{flavour.yellow.hex}",
            "green": f"#{flavour.green.hex}",
            "teal": f"#{flavour.teal.hex}",
            "sky": f"#{flavour.sky.hex}",
            "sapphire": f"#{flavour.sapphire.hex}",
            "blue": f"#{flavour.blue.hex}",
            "lavender": f"#{flavour.lavender.hex}",
            "text": f"#{flavour.text.hex}",
            "subtext1": f"#{flavour.subtext1.hex}",
            "subtext0": f"#{flavour.subtext0.hex}",
            "overlay2": f"#{flavour.overlay2.hex}",
            "overlay1": f"#{flavour.overlay1.hex}",
            "overlay0": f"#{flavour.overlay0.hex}",
            "surface2": f"#{flavour.surface2.hex}",
            "surface1": f"#{flavour.surface1.hex}",
            "surface0": f"#{flavour.surface0.hex}",
            "base": f"#{flavour.base.hex}",
            "mantle": f"#{flavour.mantle.hex}",
            "crust": f"#{flavour.crust.hex}",
        }
    )


latte = _make_theme(Flavour.latte())
frappe = _make_theme(Flavour.frappe())
macchiato = _make_theme(Flavour.macchiato())
mocha = _make_theme(Flavour.mocha())



"""
Functionality relating to Catppuccin flavours.
A flavour is a collection of colours.
"""
from dataclasses import dataclass

from catppuccin.colour import Colour


@dataclass(frozen=True)
class Flavour:  # pylint: disable=too-many-instance-attributes
    """All the colours in a flavour of Catppuccin."""

    rosewater: Colour
    flamingo: Colour
    pink: Colour
    mauve: Colour
    red: Colour
    maroon: Colour
    peach: Colour
    yellow: Colour
    green: Colour
    teal: Colour
    sky: Colour
    sapphire: Colour
    blue: Colour
    lavender: Colour
    text: Colour
    subtext1: Colour
    subtext0: Colour
    overlay2: Colour
    overlay1: Colour
    overlay0: Colour
    surface2: Colour
    surface1: Colour
    surface0: Colour
    base: Colour
    mantle: Colour
    crust: Colour

    @staticmethod
    def latte() -> "Flavour":
        """Latte flavoured Catppuccin."""
        return Flavour(
            rosewater=Colour(220, 138, 120),
            flamingo=Colour(221, 120, 120),
            pink=Colour(234, 118, 203),
            mauve=Colour(136, 57, 239),
            red=Colour(210, 15, 57),
            maroon=Colour(230, 69, 83),
            peach=Colour(254, 100, 11),
            yellow=Colour(223, 142, 29),
            green=Colour(64, 160, 43),
            teal=Colour(23, 146, 153),
            sky=Colour(4, 165, 229),
            sapphire=Colour(32, 159, 181),
            blue=Colour(30, 102, 245),
            lavender=Colour(114, 135, 253),
            text=Colour(76, 79, 105),
            subtext1=Colour(92, 95, 119),
            subtext0=Colour(108, 111, 133),
            overlay2=Colour(124, 127, 147),
            overlay1=Colour(140, 143, 161),
            overlay0=Colour(156, 160, 176),
            surface2=Colour(172, 176, 190),
            surface1=Colour(188, 192, 204),
            surface0=Colour(204, 208, 218),
            base=Colour(239, 241, 245),
            mantle=Colour(230, 233, 239),
            crust=Colour(220, 224, 232),
        )

    @staticmethod
    def frappe() -> "Flavour":
        """Frappé flavoured Catppuccin."""
        return Flavour(
            rosewater=Colour(242, 213, 207),
            flamingo=Colour(238, 190, 190),
            pink=Colour(244, 184, 228),
            mauve=Colour(202, 158, 230),
            red=Colour(231, 130, 132),
            maroon=Colour(234, 153, 156),
            peach=Colour(239, 159, 118),
            yellow=Colour(229, 200, 144),
            green=Colour(166, 209, 137),
            teal=Colour(129, 200, 190),
            sky=Colour(153, 209, 219),
            sapphire=Colour(133, 193, 220),
            blue=Colour(140, 170, 238),
            lavender=Colour(186, 187, 241),
            text=Colour(198, 208, 245),
            subtext1=Colour(181, 191, 226),
            subtext0=Colour(165, 173, 206),
            overlay2=Colour(148, 156, 187),
            overlay1=Colour(131, 139, 167),
            overlay0=Colour(115, 121, 148),
            surface2=Colour(98, 104, 128),
            surface1=Colour(81, 87, 109),
            surface0=Colour(65, 69, 89),
            base=Colour(48, 52, 70),
            mantle=Colour(41, 44, 60),
            crust=Colour(35, 38, 52),
        )

    @staticmethod
    def macchiato() -> "Flavour":
        """Macchiato flavoured Catppuccin."""
        return Flavour(
            rosewater=Colour(244, 219, 214),
            flamingo=Colour(240, 198, 198),
            pink=Colour(245, 189, 230),
            mauve=Colour(198, 160, 246),
            red=Colour(237, 135, 150),
            maroon=Colour(238, 153, 160),
            peach=Colour(245, 169, 127),
            yellow=Colour(238, 212, 159),
            green=Colour(166, 218, 149),
            teal=Colour(139, 213, 202),
            sky=Colour(145, 215, 227),
            sapphire=Colour(125, 196, 228),
            blue=Colour(138, 173, 244),
            lavender=Colour(183, 189, 248),
            text=Colour(202, 211, 245),
            subtext1=Colour(184, 192, 224),
            subtext0=Colour(165, 173, 203),
            overlay2=Colour(147, 154, 183),
            overlay1=Colour(128, 135, 162),
            overlay0=Colour(110, 115, 141),
            surface2=Colour(91, 96, 120),
            surface1=Colour(73, 77, 100),
            surface0=Colour(54, 58, 79),
            base=Colour(36, 39, 58),
            mantle=Colour(30, 32, 48),
            crust=Colour(24, 25, 38),
        )

    @staticmethod
    def mocha() -> "Flavour":
        """Mocha flavoured Catppuccin."""
        return Flavour(
            rosewater=Colour(245, 224, 220),
            flamingo=Colour(242, 205, 205),
            pink=Colour(245, 194, 231),
            mauve=Colour(203, 166, 247),
            red=Colour(243, 139, 168),
            maroon=Colour(235, 160, 172),
            peach=Colour(250, 179, 135),
            yellow=Colour(249, 226, 175),
            green=Colour(166, 227, 161),
            teal=Colour(148, 226, 213),
            sky=Colour(137, 220, 235),
            sapphire=Colour(116, 199, 236),
            blue=Colour(137, 180, 250),
            lavender=Colour(180, 190, 254),
            text=Colour(205, 214, 244),
            subtext1=Colour(186, 194, 222),
            subtext0=Colour(166, 173, 200),
            overlay2=Colour(147, 153, 178),
            overlay1=Colour(127, 132, 156),
            overlay0=Colour(108, 112, 134),
            surface2=Colour(88, 91, 112),
            surface1=Colour(69, 71, 90),
            surface0=Colour(49, 50, 68),
            base=Colour(30, 30, 46),
            mantle=Colour(24, 24, 37),
            crust=Colour(17, 17, 27),
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['catppuccin', 'catppuccin.extras']

package_data = \
{'': ['*']}

extras_require = \
{'pygments': ['pygments>=2.13.0,<3.0.0'], 'rich': ['rich>=13.3.5,<14.0.0']}

entry_points = \
{'pygments.styles': ['catppuccin-frappe = '
                     'catppuccin.extras.pygments:FrappeStyle',
                     'catppuccin-latte = catppuccin.extras.pygments:LatteStyle',
                     'catppuccin-macchiato = '
                     'catppuccin.extras.pygments:MacchiatoStyle',
                     'catppuccin-mocha = '
                     'catppuccin.extras.pygments:MochaStyle']}

setup_kwargs = {
    'name': 'catppuccin',
    'version': '1.3.2',
    'description': '🐍 Soothing pastel theme for Python.',
    'long_description': '<h3 align="center">\n\t<img src="https://raw.githubusercontent.com/catppuccin/catppuccin/main/assets/logos/exports/1544x1544_circle.png" width="100" alt="Logo"/><br/>\n\t<img src="https://raw.githubusercontent.com/catppuccin/catppuccin/main/assets/misc/transparent.png" height="30" width="0px"/>\n\tCatppuccin for <a href="https://www.python.org/">Python</a>\n\t<img src="https://raw.githubusercontent.com/catppuccin/catppuccin/main/assets/misc/transparent.png" height="30" width="0px"/>\n</h3>\n\n<p align="center">\n\t<a href="https://github.com/catppuccin/python/stargazers"><img src="https://img.shields.io/github/stars/catppuccin/python?colorA=363a4f&colorB=b7bdf8&style=for-the-badge"></a>\n\t<a href="https://github.com/catppuccin/python/issues"><img src="https://img.shields.io/github/issues/catppuccin/python?colorA=363a4f&colorB=f5a97f&style=for-the-badge"></a>\n\t<a href="https://github.com/catppuccin/python/contributors"><img src="https://img.shields.io/github/contributors/catppuccin/python?colorA=363a4f&colorB=a6da95&style=for-the-badge"></a>\n</p>\n\n## Installation\n\nInstall with `pip` or your preferred dependency management tool.\n\n```bash\npip install catppuccin\n```\n\n## Usage\n\n```python\n>>> from catppuccin import Flavour\n>>> Flavour.latte().mauve.hex\n\'8839ef\'\n>>> Flavour.mocha().teal.rgb\n(148, 226, 213)\n```\n\n`Flavour` is a [`dataclass`](https://docs.python.org/3/library/dataclasses.html),\nso you can inspect its fields to get access to the full set of colour names and values:\n\n```python\n>>> from dataclasses import fields\n>>> flavour = Flavour.frappe()\n>>> for field in fields(flavour):\n        colour = getattr(flavour, field.name)\n        print(f"{field.name}: #{colour.hex}")\nrosewater: #f2d5cf\nflamingo: #eebebe\npink: #f4b8e4\n...\nbase: #303446\nmantle: #292c3c\ncrust: #232634\n```\n\n## Pygments Styles\n\nThis package provides a Pygments style for each of the four Catppuccin flavours.\n\nInstall Catppuccin with the `pygments` feature to include the relevant dependencies:\n\n```bash\npip install catppuccin[pygments]\n```\n\nThe styles are registered as importlib entrypoints, which allows Pygments to\nfind them by name:\n\n```python\n>>> from pygments.styles import get_style_by_name\n>>> get_style_by_name("catppuccin-frappe")\ncatppuccin.extras.pygments.FrappeStyle\n```\n\nThe following style names are available:\n\n - `catppuccin-latte`\n - `catppuccin-frappe`\n - `catppuccin-macchiato`\n - `catppuccin-mocha`\n\nThey can also be accessed by directly importing them:\n\n```python\nfrom catppuccin.extras.pygments import MacchiatoStyle\n```\n\n## Contribution\n\nIf you are looking to contribute, please read through our\n[CONTRIBUTING.md](https://github.com/catppuccin/.github/blob/main/CONTRIBUTING.md)\nfirst!\n\n### Development\n\nThis project is maintained with [Poetry](https://python-poetry.org). If you\ndon\'t have Poetry yet, you can install it using the [installation\ninstructions](https://python-poetry.org/docs/#installation).\n\nInstall the project\'s dependencies including extras:\n\n```bash\npoetry install -E pygments\n```\n\n#### Code Standards\n\nBefore committing changes, it is recommended to run the following tools to\nensure consistency in the codebase.\n\n```bash\nisort .\nblack .\npylint catppuccin\nmypy .\npytest --cov catppuccin\n```\n\nThese tools are all installed as part of the `dev` dependency group with\nPoetry. You can use `poetry shell` to automatically put these tools in your\npath.\n\n\n## 💝 Thanks to\n\n-   [backwardspy](https://github.com/backwardspy)\n\n&nbsp;\n\n<p align="center">\n\t<img src="https://raw.githubusercontent.com/catppuccin/catppuccin/main/assets/footers/gray0_ctp_on_line.svg?sanitize=true" />\n</p>\n<p align="center">\n\tCopyright &copy; 2022-present <a href="https://github.com/catppuccin" target="_blank">Catppuccin Org</a>\n</p>\n<p align="center">\n\t<a href="https://github.com/catppuccin/catppuccin/blob/main/LICENSE"><img src="https://img.shields.io/static/v1.svg?style=for-the-badge&label=License&message=MIT&logoColor=d9e0ee&colorA=363a4f&colorB=b7bdf8"/></a>\n</p>\n',
    'author': 'backwardspy',
    'author_email': 'backwardspy@pigeon.life',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)



"""
Pygments styles for all Catppuccin flavours.
"""
from typing import Dict

from pygments.style import Style
from pygments.token import (
    Comment,
    Error,
    Generic,
    Keyword,
    Literal,
    Name,
    Number,
    Operator,
    Punctuation,
    String,
    Text,
    Token,
    _TokenType,
)

from catppuccin.flavour import Flavour


def _make_styles(flavour: Flavour) -> Dict[_TokenType, str]:
    return {
        Token: f"#{flavour.text.hex}",
        Text: f"#{flavour.text.hex}",
        Error: f"#{flavour.red.hex}",
        Keyword: f"#{flavour.mauve.hex}",
        Keyword.Constant: f"#{flavour.peach.hex}",
        Keyword.Declaration: f"#{flavour.blue.hex}",
        Keyword.Namespace: f"#{flavour.teal.hex}",
        Keyword.Pseudo: f"#{flavour.mauve.hex}",
        Keyword.Reserved: f"#{flavour.mauve.hex}",
        Keyword.Type: f"#{flavour.blue.hex}",
        Name: f"#{flavour.peach.hex}",
        Name.Attribute: f"#{flavour.blue.hex}",
        Name.Constant: f"#{flavour.yellow.hex}",
        Name.Decorator: f"#{flavour.blue.hex}",
        Name.Function: f"#{flavour.blue.hex}",
        Name.Function.Magic: f"#{flavour.sky.hex}",
        Name.Label: f"#{flavour.blue.hex}",
        Name.Tag: f"#{flavour.mauve.hex}",
        Literal: f"#{flavour.text.hex}",
        String: f"#{flavour.green.hex}",
        Number: f"#{flavour.peach.hex}",
        Punctuation: f"#{flavour.text.hex}",
        Operator: f"#{flavour.sky.hex}",
        Comment: f"#{flavour.overlay0.hex}",
        Generic.Heading: f"#{flavour.blue.hex} bold",
    }


class LatteStyle(Style):  # pylint: disable=too-few-public-methods
    """Catppuccin Latte pygments style."""

    _flavour = Flavour.latte()

    background_color = f"#{_flavour.base.hex}"
    line_number_background_color = f"#{_flavour.mantle.hex}"
    line_number_color = f"#{_flavour.text.hex}"

    styles = _make_styles(_flavour)


class FrappeStyle(Style):  # pylint: disable=too-few-public-methods
    """Catppuccin Frappé pygments style."""

    _flavour = Flavour.frappe()

    background_color = f"#{_flavour.base.hex}"
    line_number_background_color = f"#{_flavour.mantle.hex}"
    line_number_color = f"#{_flavour.text.hex}"

    styles = _make_styles(_flavour)


class MacchiatoStyle(Style):  # pylint: disable=too-few-public-methods
    """Catppuccin Macchiato pygments style."""

    _flavour = Flavour.macchiato()

    background_color = f"#{_flavour.base.hex}"
    line_number_background_color = f"#{_flavour.mantle.hex}"
    line_number_color = f"#{_flavour.text.hex}"

    styles = _make_styles(_flavour)


class MochaStyle(Style):  # pylint: disable=too-few-public-methods
    """Catppuccin Mocha pygments style."""

    _flavour = Flavour.mocha()

    background_color = f"#{_flavour.base.hex}"
    line_number_background_color = f"#{_flavour.mantle.hex}"
    line_number_color = f"#{_flavour.text.hex}"

    styles = _make_styles(_flavour)
